// $ANTLR 2.7.6 (20060919): "expressions.g" -> "ExpressionsParser.java"$

/*
 * Copyright (c) 2004-2005 Universite de Nantes (LINA)
 * Copyright (c) 2005-2006 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Pierre-Charles David <pcdavid@gmail.com>
 */
package org.objectweb.wildcat.expressions;

import java.util.*;
import org.objectweb.wildcat.Context;
import org.objectweb.wildcat.Path;
import org.objectweb.wildcat.expressions.functions.Function;

import antlr.TokenBuffer;
import antlr.TokenStreamException;
import antlr.TokenStreamIOException;
import antlr.ANTLRException;
import antlr.LLkParser;
import antlr.Token;
import antlr.TokenStream;
import antlr.RecognitionException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.ParserSharedInputState;
import antlr.collections.impl.BitSet;

public class ExpressionsParser extends antlr.LLkParser       implements ExpressionsParserTokenTypes
 {

	private String unquote(String str) {
		str = str.substring(1, str.length() - 1); // removes leadding & ending quotes
		str = str.replaceAll("\\\\'", "'");       // unquotes quotes single-quotes
		str = str.replaceAll("\\\\\"", "\"");     // unquotes quotes double-quotes
		return str;
	}

protected ExpressionsParser(TokenBuffer tokenBuf, int k) {
  super(tokenBuf,k);
  tokenNames = _tokenNames;
}

public ExpressionsParser(TokenBuffer tokenBuf) {
  this(tokenBuf,3);
}

protected ExpressionsParser(TokenStream lexer, int k) {
  super(lexer,k);
  tokenNames = _tokenNames;
}

public ExpressionsParser(TokenStream lexer) {
  this(lexer,3);
}

public ExpressionsParser(ParserSharedInputState state) {
  super(state,3);
  tokenNames = _tokenNames;
}

/**
 * Top-level production. Alias for orExpression.
 */
	public final  Expression  expression() throws RecognitionException, TokenStreamException {
		 Expression expr ;
		
		expr = null;
		
		expr=orExpression();
		return expr ;
	}
	
	public final  Expression  orExpression() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		
		result = null;
		List clauses = null;
		Expression e1 = null, e2 = null;
		
		
		e1=andExpression();
		result = e1;
		{
		_loop57:
		do {
			if ((LA(1)==OR)) {
				match(OR);
				e2=andExpression();
				
				if (clauses == null) {
				clauses = new ArrayList();
				clauses.add(e1);
				}
				clauses.add(e2);
				
			}
			else {
				break _loop57;
			}
			
		} while (true);
		}
		
		if (clauses != null) {
			java.util.Collections.reverse(clauses);
		result = new OrExpression((Expression[]) clauses.toArray(new Expression[clauses.size()]));
		}
		
		return result ;
	}
	
	public final  Expression  andExpression() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		
		result = null;
		List clauses = null;
		Expression e1 = null, e2 = null;
		
		
		e1=compExpression();
		result = e1;
		{
		_loop60:
		do {
			if ((LA(1)==AND)) {
				match(AND);
				e2=compExpression();
				
				if (clauses == null) {
				clauses = new ArrayList();
				clauses.add(e1);
				}
				clauses.add(e2);
				
			}
			else {
				break _loop60;
			}
			
		} while (true);
		}
		
		if (clauses != null) {
			java.util.Collections.reverse(clauses);
		result = new AndExpression((Expression[]) clauses.toArray(new Expression[clauses.size()]));
		}
		
		return result ;
	}
	
	public final  Expression  compExpression() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		
			result = null;
			Expression right = null;
		
		
		result=plusExpression();
		{
		_loop63:
		do {
			switch ( LA(1)) {
			case EQUAL:
			{
				match(EQUAL);
				right=plusExpression();
				result = new CallExpression("equals", result, right);
				break;
			}
			case NEQUAL:
			{
				match(NEQUAL);
				right=plusExpression();
				result = new CallExpression("not", new CallExpression("equals", result, right));
				break;
			}
			case LESS:
			{
				match(LESS);
				right=plusExpression();
				result = new CallExpression("less", result, right);
				break;
			}
			case GREATER:
			{
				match(GREATER);
				right=plusExpression();
				result = new CallExpression("not", new OrExpression(new CallExpression("less", result, right),
					                                                    new CallExpression("equals", result, right)));
				break;
			}
			case LEQ:
			{
				match(LEQ);
				right=plusExpression();
				result = new OrExpression(new CallExpression("less", result, right),
					                          new CallExpression("equals", result, right));
				break;
			}
			case GEQ:
			{
				match(GEQ);
				right=plusExpression();
				result = new CallExpression("not", new CallExpression("less", result, right));
				break;
			}
			default:
			{
				break _loop63;
			}
			}
		} while (true);
		}
		return result ;
	}
	
	public final  Expression  plusExpression() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		
			result = null;
			Expression right = null;
		
		
		result=multiplyExpression();
		{
		_loop66:
		do {
			switch ( LA(1)) {
			case PLUS:
			{
				match(PLUS);
				right=multiplyExpression();
				result = new CallExpression("add", result, right);
				break;
			}
			case MINUS:
			{
				match(MINUS);
				right=multiplyExpression();
				result = new CallExpression("substract", result, right);
				break;
			}
			default:
			{
				break _loop66;
			}
			}
		} while (true);
		}
		return result ;
	}
	
	public final  Expression  multiplyExpression() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		
			result = null;
			Expression right = null;
		
		
		result=unaryExpression();
		{
		_loop69:
		do {
			switch ( LA(1)) {
			case STAR:
			{
				match(STAR);
				right=unaryExpression();
				result = new CallExpression("multiply", result, right);
				break;
			}
			case DIV:
			{
				match(DIV);
				right=unaryExpression();
				result = new CallExpression("divide", result, right);
				break;
			}
			default:
			{
				break _loop69;
			}
			}
		} while (true);
		}
		return result ;
	}
	
	public final  Expression  unaryExpression() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		result = null;
		
		switch ( LA(1)) {
		case NAME:
		case NUMBER:
		case STRING:
		case SLASH:
		case LPAREN:
		case DOLLAR:
		case SHARP:
		{
			result=atom();
			break;
		}
		case MINUS:
		{
			match(MINUS);
			result=atom();
			result = new CallExpression("substract", new ConstantExpression(0), result);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return result ;
	}
	
	public final  Expression  atom() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		result = null;
		
		switch ( LA(1)) {
		case NUMBER:
		case STRING:
		{
			result=literal();
			break;
		}
		case DOLLAR:
		{
			result=variable();
			break;
		}
		case LPAREN:
		{
			result=paren();
			break;
		}
		case NAME:
		{
			result=function();
			break;
		}
		case SLASH:
		case SHARP:
		{
			result=ctxpath();
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return result ;
	}
	
	public final  Expression  literal() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		Token  n = null;
		Token  s = null;
		result = null;
		
		switch ( LA(1)) {
		case NUMBER:
		{
			n = LT(1);
			match(NUMBER);
			result = new ConstantExpression(Double.valueOf(n.getText()));
			break;
		}
		case STRING:
		{
			s = LT(1);
			match(STRING);
			result = new ConstantExpression(unquote(s.getText()));
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		return result ;
	}
	
/**
 * $varName => (variable "varName")
 */
	public final  Expression  variable() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		Token  n = null;
		result = null;
		
		match(DOLLAR);
		n = LT(1);
		match(NAME);
		result = new VariableExpression(n.getText());
		return result ;
	}
	
/**
 * ( e ) => e
 */
	public final  Expression  paren() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		result = null;
		
		match(LPAREN);
		result=expression();
		match(RPAREN);
		return result ;
	}
	
/**
 * f(a1, a2 ...) => (call "f" a1 a2 ...)
 */
	public final  Expression  function() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		Token  func = null;
		
			result = null;
			List args = null;
		
		
		func = LT(1);
		match(NAME);
		match(LPAREN);
		{
		switch ( LA(1)) {
		case NAME:
		case NUMBER:
		case STRING:
		case MINUS:
		case SLASH:
		case LPAREN:
		case DOLLAR:
		case SHARP:
		{
			args=arguments();
			break;
		}
		case RPAREN:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		match(RPAREN);
		
		result = new CallExpression(func.getText(), (Expression[]) args.toArray(new Expression[args.size()]));
		
		return result ;
	}
	
/**
 * /steps/to/resource#attribute
 */
	public final  Expression  ctxpath() throws RecognitionException, TokenStreamException {
		 Expression result ;
		
		Token  step = null;
		Token  attr = null;
		
			result = null;
			StringBuilder str = new StringBuilder();
		
		
		{
		_loop82:
		do {
			if ((LA(1)==SLASH)) {
				match(SLASH);
				step = LT(1);
				match(NAME);
				str.append("/").append(step.getText());
			}
			else {
				break _loop82;
			}
			
		} while (true);
		}
		match(SHARP);
		attr = LT(1);
		match(NAME);
		str.append("#").append(attr.getText());
		
			   Path path = Context.createPath(str.toString());
		result = new CallExpression("attribute", new ConstantExpression(path), new VariableExpression("_context"));
		
		return result ;
	}
	
	public final  List  arguments() throws RecognitionException, TokenStreamException {
		 List args ;
		
		
			args = new ArrayList();
			Expression e1 = null, e2 = null;
		
		
		e1=expression();
		args.add(e1);
		{
		_loop79:
		do {
			if ((LA(1)==COMMA)) {
				match(COMMA);
				e2=expression();
				args.add(e2);
			}
			else {
				break _loop79;
			}
			
		} while (true);
		}
		return args ;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"\"and\"",
		"\"or\"",
		"\"div\"",
		"NAME",
		"NUMBER",
		"STRING",
		"PLUS",
		"MINUS",
		"STAR",
		"SLASH",
		"EQUAL",
		"NEQUAL",
		"LESS",
		"GREATER",
		"LEQ",
		"GEQ",
		"COMMA",
		"LPAREN",
		"RPAREN",
		"DOT",
		"DOLLAR",
		"SHARP",
		"WS",
		"SL_COMMENT",
		"SQ_STRING",
		"DQ_STRING",
		"ESC",
		"DIGIT",
		"NAME_CHAR",
		"LETTER"
	};
	
	
	}
